@@ -29,7 +29,7 @@
 #include        "cramlib.h"
 #include	<ltdl.h>
 
-static const char rcsid[]="$Id: authdaemond.c,v 1.34 2008/07/10 02:43:55 mrsam Exp $";
+static const char rcsid[]="$Id: authdaemond.c,v 1.35 2009/06/27 15:51:48 mrsam Exp $";
 
 #ifndef	SOMAXCONN
 #define	SOMAXCONN	5
@@ -848,7 +848,7 @@ char	*p;
 
 static int sighup_pipe= -1;
 
-static RETSIGTYPE sighup(int n)
+static void sighup(int n)
 {
 	if (sighup_pipe >= 0)
 	{
@@ -856,14 +856,11 @@ static RETSIGTYPE sighup(int n)
 		sighup_pipe= -1;
 	}
 	signal(SIGHUP, sighup);
-#if	RETSIGTYPE != void
-	return (1);
-#endif
 }
 
 static int sigterm_received=0;
 
-static RETSIGTYPE sigterm(int n)
+static void sigterm(int n)
 {
 	sigterm_received=1;
 	if (sighup_pipe >= 0)
@@ -876,10 +873,6 @@ static RETSIGTYPE sigterm(int n)
 		kill(0, SIGTERM);
 		_exit(0);
 	}
-
-#if	RETSIGTYPE != void
-	return (0);
-#endif
 }
 
 static int startchildren(int *pipefd)
