@@ -2,7 +2,7 @@
 #define	authsaslclient_h
 
 /*
-** Copyright 2000 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2008 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -16,9 +16,17 @@
 extern "C" {
 #endif
 
-static const char authsaslclient_h_rcsid[]="$Id: authsaslclient.h,v 1.5 2005/02/21 03:34:19 mrsam Exp $";
+static const char authsaslclient_h_rcsid[]="$Id: authsaslclient.h,v 1.6 2008/07/10 02:43:55 mrsam Exp $";
 
 
+#define SASL_LIST \
+	SASL("EXTERNAL", NO_SERVER_FUNC(), authsaslclient_external) \
+	SASL("PLAIN", SERVER_FUNC(authsasl_plain), authsaslclient_plain) \
+	SASL("LOGIN", SERVER_FUNC(authsasl_login), authsaslclient_login) \
+	SASL("CRAM-MD5", SERVER_FUNC(authsasl_cram), authsaslclient_crammd5) \
+	SASL("CRAM-SHA1", SERVER_FUNC(authsasl_cram), authsaslclient_cramsha1) \
+	SASL("CRAM-SHA256", SERVER_FUNC(authsasl_cram), authsaslclient_cramsha256)
+
 /* A list of SASL client functions */
 
 struct authsaslclientlist_info {
@@ -28,6 +36,7 @@ struct authsaslclientlist_info {
 
 extern int authsaslclient_login(const struct authsaslclientinfo *);
 extern int authsaslclient_plain(const struct authsaslclientinfo *);
+extern int authsaslclient_external(const struct authsaslclientinfo *);
 extern int authsaslclient_crammd5(const struct authsaslclientinfo *);
 extern int authsaslclient_cramsha1(const struct authsaslclientinfo *);
 extern int authsaslclient_cramsha256(const struct authsaslclientinfo *);
