@@ -1,17 +1,13 @@
-/* $Id: authsaslclientcrammd5.c,v 1.2 2004/10/21 00:10:49 mrsam Exp $ */
+/* $Id: authsaslclientcrammd5.c,v 1.3 2008/07/10 02:43:55 mrsam Exp $ */
 
 /*
-** Copyright 2000 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2008 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 #include	"courier_auth_config.h"
 #include	"courierauthsasl.h"
-
-#if	HAVE_HMACLIB
 #include	"libhmac/hmac.h"
-#endif
-
 #include	"authsaslclient.h"
 #include	<stdlib.h>
 #include	<stdio.h>
@@ -19,8 +15,6 @@
 #include	<string.h>
 #include	<errno.h>
 
-#if	HAVE_HMACLIB
-
 extern int authsaslclient_cram(const struct authsaslclientinfo *info,
 				const char *p,
 				const struct hmac_hashinfo *);
@@ -32,12 +26,3 @@ const char *p=(*info->start_conv_func)("CRAM-MD5", NULL, info->conv_func_arg);
 	if (!p) return (AUTHSASL_CANCELLED);
 	return ( authsaslclient_cram(info, p, &hmac_md5));
 }
-
-#else
-int authsaslclient_crammd5(const struct authsaslclientinfo *info)
-{
-	return (AUTHSASL_NOMETHODS);
-}
-
-#endif
-
