@@ -1,17 +1,13 @@
-/* $Id: authsaslclientcramsha1.c,v 1.3 2005/02/21 03:34:19 mrsam Exp $ */
+/* $Id: authsaslclientcramsha1.c,v 1.4 2008/07/10 02:43:55 mrsam Exp $ */
 
 /*
-** Copyright 2000 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2008 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 #include	"courier_auth_config.h"
 #include	"courierauthsasl.h"
-
-#if	HAVE_HMACLIB
 #include	"libhmac/hmac.h"
-#endif
-
 #include	"authsaslclient.h"
 #include	<stdlib.h>
 #include	<stdio.h>
@@ -19,8 +15,6 @@
 #include	<string.h>
 #include	<errno.h>
 
-#if	HAVE_HMACLIB
-
 extern int authsaslclient_cram(const struct authsaslclientinfo *,
 				const char *,
 				const struct hmac_hashinfo *);
@@ -41,17 +35,3 @@ const char *p=(*info->start_conv_func)("CRAM-SHA256", NULL,
 	if (!p) return (AUTHSASL_CANCELLED);
 	return ( authsaslclient_cram(info, p, &hmac_sha256));
 }
-
-#else
-int authsaslclient_cramsha1(const struct authsaslclientinfo *info)
-{
-	return (AUTHSASL_NOMETHODS);
-}
-
-int authsaslclient_cramsha256(const struct authsaslclientinfo *info)
-{
-	return (AUTHSASL_NOMETHODS);
-}
-
-#endif
-
