@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2007 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -13,7 +13,7 @@
 #include	<stdio.h>
 #include	<errno.h>
 
-static const char rcsid[]="$Id: userdb2.c,v 1.4 2004/04/18 15:54:39 mrsam Exp $";
+static const char rcsid[]="$Id: userdb2.c,v 1.6 2007/04/14 03:20:55 mrsam Exp $";
 
 extern int userdb_debug_level;
 
@@ -27,8 +27,9 @@ size_t	l;
 
 	if (dbobj_open(&d, sh, "R"))
 	{
-		userdb_debug_level && fprintf(stderr,
-			"DEBUG: userdbshadow: unable to open %s\n", sh);
+		if (userdb_debug_level)
+			fprintf(stderr,
+				"DEBUG: userdbshadow: unable to open %s\n", sh);
 		return (0);
 	}
 
@@ -36,14 +37,19 @@ size_t	l;
 	dbobj_close(&d);
 	if (!q)
 	{
-		userdb_debug_level && fprintf(stderr,
-			"DEBUG: userdbshadow: entry not found\n");
+		if (userdb_debug_level)
+			fprintf(stderr,
+				"DEBUG: userdbshadow: entry not found\n");
 		errno=ENOENT;
 		return(0);
 	}
 
 	p=malloc(l+1);
-	if (!p)	return (0);
+	if (!p)
+	{
+		free(q);
+		return (0);
+	}
 
 	if (l)	memcpy(p, q, l);
 	free(q);
